#ifndef __CHM_DATE_TIME_CLASS_H__
#define __CHM_DATE_TIME_CLASS_H__
//------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMdateTimeClass
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------
#include <CHM/CHMminimumInclude.h>
#include <time.h>

class CHMdateTimePrivate;
class CHMstring;

/** Date/Time class.
 * This class provides convenient methods to access members of
 * date/time columns in Chameleon tables.
 **/
class CHMdateTime 
{
public:
   /** Default constructor. */
   CHMdateTime();

   /** Construct a date/time object from a \c time_t value. */
   CHMdateTime(time_t Time);

   /** Constructor used in the internal implementation of Chameleon to construct a date/time 
    *  object from a handle to the underlying Chameleon date/time object. This should not be called. */
   explicit CHMdateTime(CHMdateTimeHandle Handle);

   /** Copy constructor. */
   CHMdateTime(const CHMdateTime& Orig);

   /** Destructor. */
   virtual ~CHMdateTime();

   /** Set the date/time to the value denoted by \c time_t quantity \c Time.  Returns the resulting date/time object. */
   CHMdateTime& operator=(time_t Time);

   /** Assignment operator. */
   CHMdateTime& operator=(const CHMdateTime& Orig);

   /** Get a floating point quantity that measures the number of days from midnight (12AM) 30 December 1899. For example, 
    *  6AM 1 January 1900 is represented as 2.25.  Hours and minutes are represented as fractional days, i.e., 
    *  0.25 * 24 hours = 6AM.  OLE DATE quantity.
    **/
   DATE Date() const;

   /** Get the handle for the underlying date/time object. 
    * This is part of the internal implementation of Chameleon and should not be called. */
   const CHMdateTimeHandle DateTimeHandle() const;

   /** Get the day of the month. The range of possible return values is [1..31]. */
   int Day() const;
  
   /** Get the day of the week as an integer. The range of possible return values is [1..7] where 1=Sun, 2=Mon, etc. */
   int DayOfWeek() const;

   /** Get the day of the year. The range of possible return values is [1..365] where Jan 1st = 1, etc. */
   int DayOfYear() const; 

   /** Output the current date/time to a string in the format Day/Month/Year Hour:Minute:Second (Time Zone).
    *  For example, February 28, 1977 12:34pm EST would be output as 28/02/1977 12:34:00 (zone -5). */
   CHMstring Dump() const;

   /** Get the hour of the day. The range of possible return values is [0..23]. */
   int Hour() const; 

   /** Check if \c this date/time is null. 
    * This is very useful for determining whether or not a date/time is present in a message. 
    * */
   CHMboolean IsNull() const;

   /** Get the minute of the hour. The range of possible return values is [0..59]. */
   int Minute() const; 

   /** Get the month as an integer. The range of possible return values is [1..12] where January = 1, February = 2, etc. */
   int Month() const;

   /** Get the current date/time. Static operator.  */
   static CHMdateTime Now();

   /** Get the second of the minute. The range of possible return values is [0..59]. */
   int Second() const;

   /** Get the fractional number of seconds. */
   double SecondFraction() const;

   /** Set the date to \c nYear/\c nMonth/\c nDay.
    * Returns true if the date is valid. */ 
   CHMboolean SetDate(int nYear, int nMonth, int nDay);

   /** Set the DATE to \c NewValue. See \c Date() for an explanation of the format of a DATE type. */
   void SetDate(DATE NewValue);

   /** Set the date to \c nYear/\c nMonth/<tt>nDay</tt>, and the time to \c nHour:\c nMin:\c nSec. 
    * Returns true if the date/time is valid. */ 
   CHMboolean SetDateTime(int nYear, int nMonth, int nDay,
                          int nHour, int nMin,   int nSec);

   /** Set \c this date/time to null. */
   void setNull();

   /** Set the fractional number of seconds to \c Value. */
   void SetSecondFraction(double Value);

   /** Set the time to \c nHour:\c nMin:\c nSec.
    * Returns true if the time is valid. */ 
   CHMboolean SetTime(int nHour, int nMin, int nSec);
  
   /** Set the time zone with the short integer \c Value. */
   void SetZone(short Value);
   
   /** Get the year. */
   int Year() const;

   /** Get a short integer indicating the time zone. */
   short Zone() const;

   // This function is deprecated. Use Dump instead. 
   // See manual for migration guide.
   //
   //void PrintOn(ostream& Stream) const;

private:
   CHMdateTimePrivate* pMember;
};

// This function is deprecated. Use Dump instead. 
// See manual for migration guide.
//
// ostream& operator<<(ostream& Stream, const CHMdateTime& DateTime);

#endif // end of defensive include
