#ifndef __CHM_HL7_SERVER_SOCKET_H__
#define __CHM_HL7_SERVER_SOCKET_H__

#include <CHM/CHMminimumInclude.h>
#include <CHM/CHMhl7ClientSocket.h>

struct CHMhl7ServerSocketPrivate;

/** Requests that the network thread stop and will block until it has completed.
 *
 *  This is good thing to do during the shutdown of your application if you are using
 *  these networking components.  The networking thread
    will finish all pending network events after this is called before it stops.
*/
void CHMnetStop();

// CHMnetWait is not required anymore since CHMnetStop() will block until the networking
// thread stops so this function has been removed.
//void CHMnetWait();

/** Deprecated server class for asynchronous TCP/IP communication.  This should
 * not be used for new applications.
 *
 * The CHMhl7ServerSocket represents a listener on a port that can receive
 * remote connections. It also supports sending and receiving messages
 * across these connections. The messages are automatically wrapped and 
 * unwrapped using the LLP protocol.
 * To implement a CHMhl7ServerSocket you must create a new class that inheirits 
 * from it. This way you can override the virtual methods to handle the network events.
 *
 * \deprecated
 */
class CHMhl7ServerSocket : public CHMconnectionEvents
{
public:
   /** Constructor 
       The default constructor creates a server object that's ready to listen. Call
       the listen() method to start listening.
   */
   CHMhl7ServerSocket();

   /** Destructor */
   virtual ~CHMhl7ServerSocket();

   /** Makes server listen on a specified port 
       Calling listen makes this socket start listening on the port specified. After this call
       is made, and there is no error, the socket is ready to receive incoming connecctions.
       When an incoming connnection is accepted the onNewConnection() event is called. If there
       is an error trying to listen the onError() event is called. Override these virtual methods 
       on a base class to handle these events.
       \param Port The port number to listen on.
   */
   void listen( short Port );

   /** Stops the server from listening 
       Calling this method stops the socket from listening on the port specified in the call to listen().
       Any incoming connection requests made after this method is called will fail.
   */
   void stopListening();

   /** Send a message. 
       Sends a message on the connection specified by ClientId. This call automatically wraps the message
       with the LLP header and footer.
      \param ClientId The id of the client to send a message to. The ClientId must be a valid id 
         received from CHMhl7ClientSocket or onNewConnection().
      \param Message A null terminated string representing the message that should be sent.
   */
   void sendMessage( int ClientId, const char* Message );

   /** Get the raw IP address 
      \param ClientId The id of the connection you'd like the IP of. The ClientId must be a valid id 
         received from CHMhl7ClientSocket or onNewConnection().
      \return The 32 bit integer IP address of the connection specified by ClientId.
   */
   CHMuint32 getIpAddress( int ClientId );

   /** Get the IP address as a string.
      \param ClientId The id of the connection you'd like the IP of. The ClientId must be a valid id 
         received from CHMhl7ClientSocket or onNewConnection().
      \return The IP address of the connection specified by ClientId as a string in the form "127.0.0.1".
   */
   const char* getIpAddressString( int ClientId );

   /** Gets the host name.
      This function does a lookup for the host name of the connection's IP address. If 
      no host name is found then it returns the IP address of the remote host as a string.
      \param ClientId The id of the connection you'd like the host name of. The ClientId must be a valid id 
         received from CHMhl7ClientSocket or onNewConnection().
      \return The host name of the remote host.
   */
   const char* getHostName( int ClientId );

   /** Close a connection. 
      Closes the connection specified by ClientId. If the connection has already been closed this
      function does nothing.
      \param ClientId The id of the connection you'd like to close. The ClientId must be a valid id 
         received from CHMhl7ClientSocket or onNewConnection().
   */
   void closeConnection( int ClientId );

   /** Close all connections. 
       Closes all the connections that this server socket has accepted.
   */
   void closeAllConnections();

   /** Called when there is a new connection 
       This method is called when a new connection has been accepted. The default behavior is to create a 
       CHMhl7ClientSocket. You can override this method on a base class to return your own derived
       implementation of CHMhl7ClientSocket.
       \param ClientId The id of the new connection. You can use this id to create a new CHMhl7ClientSocket
         to return.
       \return A new CHMhl7ClientSocket to handle the new connection. If you override this method you
         can return you own derived version of CHMhl7ClientSocket to handle its events properly.
   */
   virtual CHMhl7ClientSocket& onNewConnection( int ClientId );

   /** called when an error occurs
       This method is called when an error occurs with the server socket such as not being able to listen on a port. 
       Errors that occur on a connection are not handled here but with the onTransportError() event. You can override 
       this method on a base class to return your own derived implementation of CHMhl7ClientSocket.
       \param ErrorId The error code. This is a number that identifies the error type.
       \param ErrorMessage A null terminated string with the error message.
   */
   virtual void onError( int ErrorId, const char* ErrorMessage ){}

private:
   friend class CHMhl7ClientSocket;
   static void ensureInitialized();

   CHMhl7ServerSocketPrivate* pMember;
   
   // not allowed
   CHMhl7ServerSocket(const CHMhl7ServerSocket& Orig);
   CHMhl7ServerSocket& operator=(const CHMhl7ServerSocket& Orig);
};

#endif // end of defensive include
