#ifndef _CHM_STRING_H_
#define _CHM_STRING_H_
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMstring
//
// Description:String class derived from work by Lynn Prentice
//       and Ian Farquharson.
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------
// This is super efficient string class used by HL7 Chameleon
#include <CHM/CHMminimumInclude.h>

#define COPYONWRITE

#ifdef __MSC__
   const size_t NPOS = npos;
#endif

//=============================================================================

class CHMstringRef;

/**
* Custom string class.  
* Because Chameleon is cross platform
* is it necessary to use a string class which is independent of
* the various compilers, operating systems, and C++ class libraries
* we support.  This string is efficiently implemented with reference
* counting.
**/
class CHMstring 
{
public:
   // Constructors
   /** Construct an empty string. */ 
   CHMstring(void);
   
   /** Copy constructor. */
   CHMstring(const CHMstring& s);   
   
   /** Construct a string from a substring of the string \c s starting at index \c orig for \c n characters. 
    * <tt>npos = (size_t) - 1 </tt>.*/
   CHMstring(const CHMstring& s, const size_t& orig, const size_t& n = npos);   
   
   /** Construct a string from a null terminated string \c cp. */
   CHMstring(const CHMchar* cp);   
   
   /** Construct a string from a substring of a null terminated string \c cp starting at index \c orig 
    * for \c n characters. <tt>npos = (size_t) - 1 </tt>. */
   CHMstring(const CHMchar* cp, const size_t& orig, const size_t& n = npos );   
   
   /** Construct a string from the character \c c. */
   CHMstring(CHMchar c);

   /** Construct a string that consists of \c n characters of type \c c. */
   CHMstring(size_t n, CHMchar c);
   
   /** Destructor. */
   ~CHMstring(void); 

#ifdef CHM_UNICODE_ON
   CHMstring& operator=(const char* pValue);
#endif  // CHM_UNICODE_ON
   
   /** Assignment operator. */
   CHMstring& operator =(const CHMstring& s);
   
   /** Concatenation operator for adding a null terminated string to \c this string. */
   CHMstring& operator+=(const CHMchar* cp);
   
   /** Concatenation operator for adding another string to \c this string. */
   CHMstring& operator+=(const CHMstring &s);   
    
   /** Const operator <tt>[pos]</tt> returns the character at index \c pos in \c this string. */
   const CHMchar operator[](const size_t& pos) const;
   
   /** Non-const operator <tt>[pos]</tt> returns the character at index \c pos in \c this string. */
   CHMchar& operator[](const size_t& pos);
  
   /** 
    * Used as an argument for the strip function to
    * specify its behavior.
    **/
   enum StripType 
   { 
      Leading,  /*!< Strip leading characters only. */ 
      Trailing, /*!< Strip trailing characters only. */
      Both      /*!< Strip both leading and trailing characters. */
   };
   
   /** Append the null terminated string \c cp of size \c n to \c this string at index \c orig.  
    *  Returns the resulting string. */
   CHMstring& append(const CHMchar* cp,
                     const size_t& orig = 0,
                     const size_t& n = npos);
   
   /** Append the first \c SizeOfBuffer characters from \c Buffer to \c this string.  
    * \c Buffer is not necessarily null terminated.
    * Returns the resulting string.
    */
   CHMstring& append(size_t SizeOfBuffer, 
                     const CHMchar* Buffer);
   
   /** Append string \c s of size \c n to \c this string at index \c orig. Returns the resulting string. */
   CHMstring& append(const CHMstring& s,
                     const size_t& orig = 0,
                     const size_t& n = npos);
   
   /** Change \c this string to consist of an array of \c n characters of type \c Filler. */
   void assign(size_t n, CHMchar Filler);
   
   /** Get a const CHMchar pointer to the buffer
    * of \c this string object.  The caller is not allowed
    * to write to this buffer since other string objects
    * may be referencing it.
    **/
   const CHMchar* c_str(void) const;
     
   /** Clear \c this string.   */
   void clear(void);
      
   /** Compare \c this string for equality to the null terminated string \c Compare.  This method invokes the 
    *  standard C \c strcmp function.  It returns 0 if the strings are equal. */
   int compare(const CHMchar* Compare) const;
   
   /** Compare \c this string for equality to another string \c s.  This method invokes the standard C \c strcmp 
    *  function.  It returns 0 if the strings are equal. */
   int compare(const CHMstring& s) const;
      
   /** Find the first instance of string \c s from index \c pos in \c this string.  Returns the index 
    *  of the string if found; otherwise, returns <tt>npos = (size_t)-1 </tt>. */
   const size_t find(const CHMstring& s, const size_t& pos=0) const;
  
   /** Get a non-const CHMchar pointer to the buffer of \c this string.
    * The caller is allowed to write
    * to the buffer since this call ensures that no other strings
    * reference the buffer. */
   CHMchar* get_buffer(void); 
     
   /** Insert the null terminated string \c cp of length \c n starting at index \c orig into \c this string at
    *  index \c pos. Returns the resulting string. */
   CHMstring & insert(size_t pos,
                      const CHMchar* cp,
                      const size_t& orig = 0,
                      const size_t& n = npos );
                      
   /** Insert the string \c s of length \c n starting at index \c orig into \c this string at index \c pos. 
    *  Returns the resulting string. */
   CHMstring & insert(size_t pos,
                      const CHMstring &s,
                      const size_t& orig = 0,
                      const size_t& n = npos );

   /** Check if \c this string is empty. */
   const CHMboolean is_null(void) const;    
      
   /** Prepend the null terminated string \c cp of size \c n to \c this string at index \c orig. 
    *  Returns the resulting string. */
   CHMstring& prepend(const CHMchar* cp,
            const size_t& orig = 0,
            const size_t& n = npos );
   
   /** Prepend the string \c s of size \c n to \c this string at index \c orig. Returns the resulting string. */
   CHMstring& prepend(const CHMstring &s,
                      const size_t& orig = 0,
                      const size_t& n = npos);
    
   /** Remove the substring starting at index \c pos of length \c n from \c this string. Returns the resulting string.*/
   CHMstring& remove(const size_t& pos, const size_t& n=npos);
   
   /** Replace the substring of size \c n1 starting at index \c pos with string \c s of size \c n2 starting 
    *  at index \c orig. */
   CHMstring& replace(const size_t& pos,
                      const size_t& n1,
                      const CHMstring& s,
                      const size_t& orig = 0,
                      const size_t& n2 = npos );   
      
   /** Find the first instance of string \c s searching backwards through \c this string from index \c pos.  
    *  Returns the index of the string if found; otherwise, returns <tt>npos = (size_t)-1 </tt>. */
   const size_t rfind(const CHMstring& s, const size_t& pos) const;
   
   /** Find the first instance of string \c s searching backwards through \c this string.  
    *  Returns the index of the string if found; otherwise, returns <tt>npos = (size_t)-1 </tt>. */
   const size_t rfind(const CHMstring& s) const;
  
   /** Get the length of \c this string. */
   const size_t& size(void) const;
   
   /** Strip char \c c from the beginning or the end or from both ends of \c this string depending on the value 
    *  of the enumerator \c StripType. Returns the resulting string. */
   CHMstring strip(const StripType& s = Trailing, const CHMchar& c = ' ') const;
   
   /** Strip all copies of char \c c from \c this string. */
   void stripAll(const CHMchar& c = ' ');
   
   /** Substitute characters of type \c pTarget with those of \c pReplacement.  */
   void subChar(const CHMchar pTarget,
                const CHMchar pReplacement);

   /** Get the substring at index \c pos of \c n characters in length or until the end of the original string has 
    *  been reached. */
   CHMstring substr(const size_t& pos, const size_t& n=npos) const;

protected:
   const CHMboolean valid_element(const size_t& pos ) const;
   const CHMboolean valid_index(const size_t& pos ) const;

private:
   // provided for display purposes only...
   const CHMchar* Current;
   CHMstringRef* pReference;

private:
   const size_t find_index(const CHMchar*,
                           const size_t& start) const;
   const size_t rfind_index(const CHMchar*,
                           const size_t& start) const;
};

// this constructor is trivial and called frequently ...
inline CHMstring::CHMstring(void)
 : pReference(NULL), Current(NULL) {}

inline const size_t CHMstring::rfind(const CHMstring& s) const
{
   return rfind(s, size());
};

// Check to make sure a string index refers to a valid element
inline const CHMboolean CHMstring::valid_element(const size_t& n) const
{
   return n < size();
};

// Check to make sure a string index is in range
inline const CHMboolean CHMstring::valid_index(const size_t& n) const
{
   return n <= size();
};

CHMstring operator+(const CHMchar* cp, const CHMstring& s);
CHMstring operator+(const CHMstring& s, const CHMchar* cp);

inline CHMstring operator+(const CHMstring& s1, const CHMstring& s2) 
{
   return s1 + s2.c_str();
};

CHMstring operator+(const CHMchar& s1, const CHMstring& s2);
CHMstring operator+(const CHMstring& s1, const CHMchar& s2);

// comparisons...
inline int operator == (const CHMstring& s1, const CHMstring& s2) 
{
   return s1.compare(s2) == 0;
};

inline int operator == (const CHMstring& s1, const CHMchar* s2) 
{
   return s1.compare(s2) == 0;
};

inline int operator == (const CHMchar* cp, const CHMstring& s) 
{
   return CHMstring(cp).compare(s) == 0;
};

inline int operator != (const CHMstring& s1, const CHMstring& s2) 
{
   return !(s1==s2);
};

inline int operator != (const CHMstring& s, const CHMchar* cp) 
{
   return !(s==cp);
};

inline int operator != (const CHMchar* cp, const CHMstring& s) 
{
   return !(cp==s);
};

inline int operator < (const CHMstring& s1, const CHMstring& s2) 
{
   return s1.compare(s2) < 0;
};

inline int operator < (const CHMstring& s1, const CHMchar* s2 ) 
{
   return s1.compare(s2) < 0;
};

inline int operator < (const CHMchar* cp, const CHMstring& s) 
{
   return CHMstring(cp).compare(s) < 0;
};

inline int operator <= (const CHMstring& s1, const CHMstring& s2) 
{
   return s1.compare(s2) <= 0;
};

inline int operator <= (const CHMstring& s, const CHMchar* cp) 
{
   return s.compare(cp) <= 0;
};

inline int operator <= (const CHMchar* cp, const CHMstring& s) 
{
   return CHMstring(cp).compare(s) <= 0;
};

inline int operator > (const CHMstring& s1, const CHMstring& s2) 
{
   return s1.compare(s2) > 0;
};

inline int operator > (const CHMstring& s, const CHMchar* cp) 
{
   return s.compare(cp) > 0;
};

inline int operator > (const CHMchar* cp, const CHMstring& s) 
{
   return CHMstring(cp).compare(s) > 0;
};

inline int operator >= (const CHMstring& s1, const CHMstring& s2) 
{
   return s1.compare(s2) >= 0;
};

inline int operator >= (const CHMstring& s, const CHMchar* cp) 
{
   return s.compare(cp) >= 0;
};

inline int operator >= (const CHMchar* cp, const CHMstring& s) 
{
   return CHMstring(cp).compare(s) >= 0;
};

inline const CHMchar CHMstring::operator[](const size_t& pos) const 
{
#ifdef _DEBUG
   if (pos >= size())
   {
      throw "BOUNDSERROR";
   }
#endif
   return c_str()[pos];
};

inline CHMstring& CHMstring::append(const CHMstring& s,
                        const size_t& orig,
                        const size_t& n) 
{
   return append(s.c_str(), orig, n);
};

inline CHMstring& CHMstring::prepend(const CHMstring &s,
                         const size_t& orig,
                         const size_t& n) 
{
   return prepend(s.c_str(), orig, n);
};

inline CHMstring& CHMstring::insert(size_t pos,
                        const CHMstring &s,
                        const size_t& orig,
                        const size_t& n) 
{
   return insert(pos, s.c_str(), orig, n);
};

inline const CHMchar* CHMstring::c_str(void) const 
{
   if (Current == NULL)
   {
      return CHM_TEXT("");
   }
   return Current;
};

inline const CHMboolean CHMstring::is_null() const
{
   if (NULL == pReference) 
           return true;
   return size() == 0;
}

#endif
