#ifndef __CHM_TABLE_CLASS_H__
#define __CHM_TABLE_CLASS_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMtableClass
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------
#include <CHM/CHMminimumInclude.h>
#include <CHM/CHMdataType.h>
class CHMtablePrivate;
class CHMtableRow;
class CHMdateTime;
class CHMstring;

/** Table class.
 * This class gives the interface to the basic data container in
 * the Chameleon framework - a nested in-memory table.  A good way
 * to get a quick look at the contents of a Table class is to use
 * the Dump method.
 */
class CHMtable 
{
public:
   /** Construct a table. */
   CHMtable();
   explicit CHMtable(CHMtableHandle Handle);

   enum State
   {
      CHM_VALID = 0,
      CHM_PRESENT_BUT_NULL = 1,
      CHM_EMPTY = 2,
      CHM_INVALID = 3
   };

   /** Destructor. */
   virtual ~CHMtable();
      
   /** Const operator <tt>[RowIndex]</tt> returns a const pointer to a CHMtableRow object. */
   const CHMtableRow* operator[](size_t RowIndex) const;
   
   /** Non-const operator <tt>[RowIndex]</tt> returns a non-const pointer to a CHMtableRow object. */
   CHMtableRow* operator[](size_t RowIndex);

   /** Add a new row to \c this table. */
   void AddRow();
   
   /** Get the index of the column named \c ColumnName.
    * Returns <tt>npos = (size_t)-1 </tt> if the column does not exist.
    */
   size_t ColumnIndex(const char* ColumnName) const;
   
   /** Get the name of the column located at \c ColumnIndex. */
   const char* ColumnName(size_t ColumnIndex) const;
      
   /** Get the data type of the column at \c ColumnIndex.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types. */
   CHMdataType ColumnType(size_t ColumnIndex) const;
      
   /** Get the number of columns defined in \c this table. */
   size_t CountOfColumn() const;
   
   /** Get the count of rows in \c this table.
    * It's important to check this before attempting to access even
    * the first row in case there are no rows in the table.
    */
   size_t CountOfRow() const;
   
   /** Get the count of subtables in \c this table. */
   size_t CountOfSubTable() const;

   /** Output \c this table's contents to a string.
    * This is useful for diagnostics.
    */
   CHMstring Dump() const;
 
   /** Get the value of a date/time-type column.
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   const CHMdateTime GetDateTime(size_t RowIndex, size_t ColumnIndex) const;
   
   /** Get the value of a double-type column.
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   double GetDouble  (size_t RowIndex, size_t ColumnIndex) const;
   
   /** Get the value of an integer-type column.
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   long GetInteger (size_t RowIndex, size_t ColumnIndex) const;
      
   /** Get the value of a string-type column.
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   const CHMchar* GetString  (size_t RowIndex, size_t ColumnIndex) const;
   
    /** Check if the column at \c ColumnIndex is flagged as
    * a 'key' column.  
    **/
   CHMboolean IsKey(size_t ColumnIndex) const; 
      
   /** 
    * Check if \c this table corresponds to a table grammar node (true) or a table group (false)
    */
   CHMboolean IsNode() const;

   /**
    * Test the value in the cell at row \c RowIndex and column \c ColumnIndex to see if it is 
    * null, zero, empty or invalid.  
    * See http://interfaceware.com/manual/is_null.html for details.
    */
   State GetState(size_t RowIndex, size_t ColumnIndex) const;   

   /**
    * Test the value in the cell at row \c RowIndex and column \c ColumnIndex to see if it is null.
    * See http://www.interfaceware.com/manual/is_null.html for more information.
    *
    * This is an alias for GetState(...) == CHM_NULL or GetState(...) == CHM_INVALID
    *
    * It is recommended not to rely on this function (it is here only for backward compatibility). Use
    * GetState(...) instead.
    */
   CHMboolean IsNull(size_t RowIndex, size_t ColumnIndex) const;
      
   /** Set the value at row \c RowIndex and column \c ColumnIndex of a date/time-type column.  
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   void SetDateTime(size_t RowIndex, size_t ColumnIndex, const CHMdateTime& DateTime);
   
   /** Set the value at row \c RowIndex and column \c ColumnIndex of a double-type column.  
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   void SetDouble(size_t RowIndex, size_t ColumnIndex, double Value);
      
   /** Set the value at row \c RowIndex and column \c ColumnIndex of an integer-type column.  
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   void SetInteger(size_t RowIndex, size_t ColumnIndex, long Value);
      
   /**  
    * Set the value at row \c RowIndex and column \c ColumnIndex to null.
    */
   void setNull(size_t RowIndex, size_t ColumnIndex) const;
      
   /** Set the value at row \c RowIndex and column \c ColumnIndex of a string-type column.  
    * Throws an exception if the data type doesn't match.
    * See http://www.interfaceware.com/manual/table_properties.html for more information on table data types.
    */
   void SetString(size_t RowIndex, size_t ColumnIndex, const CHMchar* pValue);
   
   /** Get a pointer to the subtable located at \c SubTableIndex of the table at \c RowIndex. */
   CHMtable* SubTable(size_t RowIndex, size_t SubTableIndex) const;

   /** Get the handle of the underlying table object. 
    * This is used for the internal implementation of this Chameleon and should not be called.
    **/
   CHMtableHandle TableHandle() const;

   /** Get the name of \c this table. */
   const char* TableName() const;

private:
   CHMtablePrivate* pMember;
   CHMtable(const CHMtable& Orig);             // not allowed
   CHMtable& operator=(const CHMtable& Orig);  // not allowed
};

#endif // end of defensive include
